#!/usr/bin/env python

# KickStarter
# Version 0.0.7
# Oct 2008
# by Luca Forina
#
# Website: http://www.gtk-apps.org/content/show.php/KickStarter?content=89274
# Licence: GPL v3
#
# KickStarter is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3,
# as published by the Free Software Foundation.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.




import serial
import commands
import gtk
import gtk.glade
import sys
import time
import pynotify
import locale
import os




class kickstarter:
	def __init__ (self):
		self.locale()
		self.device_check()
		self.gui()




	def locale(self):
		getlang = os.environ['LANG']
		setlang = getlang[0:2]
		if setlang == "it":
			self.LWelcome = "Benvenuto in KickStarter!"
			self.LStatus = "Stato: Pronto"
			self.LQuality = "Qualita' del segnale: "
			self.LConnecting = "Connessione alle "
			self.LConnected = "Connesso alle "
			self.LDisconnecting = "Disconnessione alle "
			self.LNetwork = "Proprieta' di rete:"
			self.LNetworkError = "Device PPP0 non trovata o non connessa!"
			self.LIsp = "Fornitori di connnettivita' disponibili:"
			self.LSms = "Archivio messaggi SMS:"
			self.LEmpty = "Vuoto!"
			self.LModem = "Risposta del modem: "
			self.LQuery = "Interroga modem: "
			self.LNotFound1 = "Device non trovata in /dev.\r\nRiavviare il computer per corregere il problema!"
			self.LNotFound2 = "Device non trovata o occupata.\r\nSi prega di modificare le impostazioni!"
			self.LWarning = "Kickstarter - Attenzione"
			self.Lnone = "Nessuno"
			self.LCloseMsg = "Si vuole veramente uscire?"
			self.LCloseTitle = "Chiusura KickStarter"
		else:
			self.LWelcome = "Welcome to KickStarter!"
			self.LStatus = "Status: Idle"
			self.LQuality = "Signal quality: "
			self.LConnecting = "Connecting at "
			self.LConnected = "Connected at "
			self.LDisconnecting = "Disconnecting at "
			self.LNetwork = "Network Interface properties:"
			self.LNetworkError = "Device PPP0 not found or not connected!"
			self.LIsp = "Available service providers:"
			self.LSms = "Inbox SMS archive:"
			self.LEmpty = "Empty!"
			self.LModem = "Modem response: "
			self.LQuery = "Query modem: "
			self.LNotFound1 = "Device non found in /dev.\r\nRestart the pc to solve the problem!"
			self.LNotFound2 = "Device not found or busy.\r\nPlease modify settings!"
			self.LWarning = "Kickstarter - Warning"
			self.Lnone = "None"
			self.LCloseMsg = "Should I close KickStarter?"
			self.LCloseTitle = "Closing KickStarter"




	def gui(self):
		self.xml = gtk.glade.XML (sys.path[0] + "/glade/kick-gui.glade")
		self.window = self.xml.get_widget ("window1")
		self.window.set_resizable(True)
		self.window.connect('delete_event', self.on_delete_event)
		#self.window.connect("destroy", lambda w: gtk.main_quit())


		# welcome message
		text = self.xml.get_widget ("textview1").get_buffer()
		text = text.set_text("\r\n"+self.LWelcome+"\r\n")


		# set widgets properties
		query = self.xml.get_widget ("label1")
		query = query.set_label(self.LQuery)


		# stausicon menu connect
		self.tooltip_quality()
		#self.tooltip_ip()
		statusIcon = gtk.StatusIcon()
		self.menu = gtk.Menu()
		menuItem = gtk.ImageMenuItem(gtk.STOCK_CONNECT)
		menuItem.connect('activate', self.connect)
		self.menu.append(menuItem)
		menuItem = gtk.ImageMenuItem(gtk.STOCK_ABOUT)
		menuItem.connect('activate', self.about_menu_activate)
		self.menu.append(menuItem)
		menuItem = gtk.ImageMenuItem(gtk.STOCK_QUIT)
		menuItem.connect('activate', self.quit_cb, statusIcon)
		self.menu.append(menuItem)


		# stausicon menu disconnect
		self.menu2 = gtk.Menu()
		menuItem = gtk.ImageMenuItem(gtk.STOCK_DISCONNECT)
		menuItem.connect('activate', self.disconnect)
		self.menu2.append(menuItem)
		menuItem = gtk.ImageMenuItem(gtk.STOCK_ABOUT)
		menuItem.connect('activate', self.about_menu_activate)
		self.menu2.append(menuItem)
		menuItem = gtk.ImageMenuItem(gtk.STOCK_QUIT)
		menuItem.connect('activate', self.quit_cb, statusIcon)
		self.menu2.append(menuItem)


		# statusicon
		self.icontheme = gtk.icon_theme_get_default()
		if ( self.icontheme.has_icon('modem') ):
			self.icon_modem = self.icontheme.load_icon('modem', gtk.icon_size_lookup(gtk.ICON_SIZE_BUTTON)[1], 0)
		else:
			self.icon_modem = self.window.render_icon(gtk.STOCK_NETWORK, gtk.ICON_SIZE_BUTTON)

		self.statusicon = gtk.status_icon_new_from_pixbuf(self.icon_modem)
		try:
			self.statusicon.set_tooltip(self.LStatus+"\n"+self.LQuality+self.tooltip_qlt)
		except:
			pass


		if sys.platform == "linux2":
			try:
				self.gtk_screen = self.window.get_screen()
				colormap = self.gtk_screen.get_rgba_colormap()
				if colormap:
					gtk.widget_set_default_colormap(colormap)
					self.statusicon.set_visible(True)
					self.statusicon.connect('activate', self.on_activate)
					self.statusicon.connect('popup-menu', self.popup_menu_cb, self.menu)
			except:
				pass



		# signal autoconnect
		dic = {	"on_toolbutton1_clicked" : self.connect,
		"on_toolbutton2_clicked" : self.disconnect,
		"on_toolbutton3_clicked" : self.quality,
		"on_toolbutton4_clicked" : self.ifconfig,
		"on_toolbutton5_clicked" : self.sms,
		"on_toolbutton6_clicked" : self.isp,
		"on_toolbutton9_clicked" : self.preferences,
		"on_button7_clicked" : self.apply,
		"on_button8_clicked" : self.clear,
		"on_entry1_activate" : self.apply,
		"menu_connect_activate_cb" : self.connect,
		"menu_disconnect_activate_cb" : self.disconnect,
		"menu_refresh_activate_cb" : self.quality,
		"menu_network_activate_cb" : self.ifconfig,
		"menu_index_activate_cb" : self.sms,
		"menu_quit_activate_cb" : lambda wid: gtk.main_quit(),
		"menu_preferences_activate_cb" : self.preferences,
		"menu_about_activate_cb" : self.about_menu_activate }
		self.xml.signal_autoconnect (dic)
		return




	def on_activate(self, widget, data=None):
		if self.window.get_property('visible'):
			self.window.hide()
		else:
			self.window.show()




	def on_delete_event(self, widget, event, data = None):
		msgbox = gtk.MessageDialog(parent = self.window, buttons = gtk.BUTTONS_YES_NO, flags = gtk.DIALOG_MODAL, type = gtk.MESSAGE_QUESTION, message_format = self.LCloseMsg)
		msgbox.set_title(self.LCloseTitle)
		result = msgbox.run()
		msgbox.destroy()
		if result == gtk.RESPONSE_YES:
			gtk.main_quit()
			return False
		else:
			return True




	def quit_cb(self, widget, data = None):
		if data:
			data.set_visible(False)
			gtk.main_quit()




	def popup_menu_cb(self, widget, button, time, data = None):
		if button == 3:
			if data:
				data.show_all()
				data.popup(None, None, None, 3, time)
		pass




	def about_menu_activate(self,widget):
		about = gtk.glade.XML(sys.path[0] + "/glade/about.glade").get_widget("aboutdialog")
		about.show_all()
		about.run()
		about.hide_all()




	def tooltip_quality(self):
		try:
			self.device_open()
			self.device.write('AT+CSQ\r')
			self.device.readline()
			self.tooltip_qlt = self.device.readline(10)[6:]
			self.device.close()
		except:
			pass




	def tooltip_ip(self):
		self.tooltip_ip = commands.getoutput("ifconfig ppp0 | grep inet | awk '{print $2}' | sed 's/addr://'")
		if self.tooltip_ip == "ppp0: error fetching interface information: Device not found":
			self.tooltip_ip = self.Lnone




	def preferences(self,widget):
		os.popen2("gksu gedit /etc/wvdial.conf")




	def device_open(self):
		# open serial communication with the device
		try:
			self.device = serial.Serial(port='/dev/ttyUSB0',
			parity=serial.PARITY_NONE,
			bytesize=serial.EIGHTBITS,
			stopbits=serial.STOPBITS_ONE,
			timeout=1,
			xonxoff=0,
			rtscts=0,
			baudrate=460800)
		except:
			pass




	def device_check(self):
		# check for device in /dev and popup warning if necessary
		try:
			vfileName="/dev/ttyUSB0"
			self.check_init1 = os.path.exists(vfileName)
			if self.check_init1 != True:
				if pynotify.init("KickStarter"):
					n = pynotify.Notification(self.LWarning, self.LNotFound1)
					n.show()
		except:
			pass


		# try modem init and popup warning if necessary
		try:
			self.device_open()
			self.device.write('ATZ\r')
			self.device.readline()
			self.check_init2 = self.device.readline()
			self.device.close()
			if self.check_init2[0:2] != "OK":
				if pynotify.init("KickStarter"):
					n = pynotify.Notification(self.LWarning, self.LNotFound2)
					n.show()
		except:
			pass




	def connect(self,widget):
		# enable widgets
		toolbutton1 = self.xml.get_widget ("toolbutton1")
		toolbutton1.hide()
		toolbutton2 = self.xml.get_widget ("toolbutton2")
		toolbutton2.show()
		toolbutton3 = self.xml.get_widget ("toolbutton3")
		toolbutton3.set_sensitive(0)
		toolbutton5 = self.xml.get_widget ("toolbutton5")
		toolbutton5.set_sensitive(0)
		toolbutton6 = self.xml.get_widget ("toolbutton6")
		toolbutton6.set_sensitive(0)
		button7 = self.xml.get_widget ("button7")
		button7.set_sensitive(0)
		entry1 = self.xml.get_widget ("entry1")
		entry1.set_sensitive(0)


		# time
		hour = str(time.localtime()[3])
		mins = str(time.localtime()[4])


		# stausicon
		self.statusicon.set_visible(False)
		self.icon_modem = self.window.render_icon(gtk.STOCK_YES, gtk.ICON_SIZE_BUTTON)
		self.statusicon = gtk.status_icon_new_from_pixbuf(self.icon_modem)
		self.statusicon.connect('activate', self.on_activate)
		self.statusicon.connect('popup-menu', self.popup_menu_cb, self.menu2)
		self.statusicon.set_tooltip(self.LConnected+hour+":"+mins)
		self.statusicon.set_visible(True)


		# textview messages
		text = self.xml.get_widget ("textview1").get_buffer()
		text = text.set_text("\r\n"+self.LConnecting+hour+":"+mins+"\r\n")


		# pynotify message
		if pynotify.init("KickStarter"):
			n = pynotify.Notification("KickStarter", self.LConnected+hour+":"+mins)
			n.show()


		# wvdial execute
		wvdial = os.popen2("wvdial")




	def disconnect(self,widget):
		# enable widgets
		toolbutton2 = self.xml.get_widget ("toolbutton2")
		toolbutton2.hide()
		toolbutton1 = self.xml.get_widget ("toolbutton1")
		toolbutton1.show()
		toolbutton3 = self.xml.get_widget ("toolbutton3")
		toolbutton3.set_sensitive(1)
		toolbutton5 = self.xml.get_widget ("toolbutton5")
		toolbutton5.set_sensitive(1)
		toolbutton6 = self.xml.get_widget ("toolbutton6")
		toolbutton6.set_sensitive(1)
		button7 = self.xml.get_widget ("button7")
		button7.set_sensitive(1)
		entry1 = self.xml.get_widget ("entry1")
		entry1.set_sensitive(1)


		# time
		hour = str(time.localtime()[3])
		mins = str(time.localtime()[4])


		# statusicon
		self.statusicon.set_visible(False)
		if ( self.icontheme.has_icon('modem') ):
			self.icon_modem = self.icontheme.load_icon('modem', gtk.icon_size_lookup(gtk.ICON_SIZE_BUTTON)[1], 0)
		else:
			self.icon_modem = self.window.render_icon(gtk.STOCK_NETWORK, gtk.ICON_SIZE_BUTTON)
		self.statusicon = gtk.status_icon_new_from_pixbuf(self.icon_modem)
		try:
			self.statusicon.set_tooltip(self.LStatus+"\n"+self.LQuality+self.tooltip_qlt)
		except:
			pass
		self.statusicon.set_visible(True)
		self.statusicon.connect('activate', self.on_activate)
		self.statusicon.connect('popup-menu', self.popup_menu_cb, self.menu)


		# textview messages
		text = self.xml.get_widget ("textview1").get_buffer()
		text = text.set_text("\r\n"+self.LDisconnecting+hour+":"+mins+"\r\n")


		# kill wvdial and pppd
		os.popen2("killall wvdial pppd")




	def quality(self,widget):
		try:
			self.device_open()
			self.device.write('AT+CSQ\r')
			self.device.readline()
			db_qlt = self.device.readline(10)[6:]
			self.device.close()
			text = self.xml.get_widget ("textview1").get_buffer()
			text = text.set_text("\r\n"+self.LQuality+db_qlt+" db\r\n")
		except:
			pass
			text = self.xml.get_widget ("textview1").get_buffer()
			text = text.set_text("\r\n"+self.LQuality+"\r\n\r\n"+self.LNetworkError)




	def ifconfig(self,widget):
		ifconfig = commands.getoutput("ifconfig ppp0")
		if ifconfig == "ppp0: error fetching interface information: Device not found":
			text = self.xml.get_widget ("textview1").get_buffer()
			text = text.set_text("\r\n"+self.LNetwork+"\r\n\r\n"+self.LNetworkError)
		else:
			text = self.xml.get_widget ("textview1").get_buffer()
			text = text.set_text("\r\n"+self.LNetwork+"\r\n\r\n"+ifconfig)




	def sms(self,widget):
		try:
			self.device_open()
			self.device.write('AT+CMGF=1\r')
			self.device.readline()
			self.device.close()
			self.device_open()
			self.device.write('AT+CMGL="all"\r')
			msg = self.device.readlines()
			self.device.close()
			self.device_open()
			self.device.write('AT+CMGF=0\r')
			self.device.close()
			num = len(msg)
			if num < 2:
				text = self.xml.get_widget ("textview1").get_buffer()
				text = text.set_text("\r\n"+self.LSms+"\r\n\r\n"+self.LEmpty)
			else:
				text = self.xml.get_widget ("textview1").get_buffer()
				text = text.set_text("\r\n"+self.LSms+"\r\n\r\n")
				count_sms = 0
				while count_sms < num:
					text = self.xml.get_widget ("textview1").get_buffer()
					text = text.insert_at_cursor(" "+msg[count_sms])
					count_sms = count_sms + 1
		except:
			pass
			text = self.xml.get_widget ("textview1").get_buffer()
			text = text.set_text("\r\n"+self.LSms+"\r\n\r\n"+self.LNetworkError)




	def isp(self,widget):
		try:
			self.device_open()
			self.device.write('AT+COPS=?\r')
			time.sleep(15)
			self.device.readline()
			isp = self.device.readline()
			self.device.close()
			text = self.xml.get_widget ("textview1").get_buffer()
			text = text.set_text("\r\n"+self.LIsp+"\r\n\r\n"+isp)
		except:
			pass
			text = self.xml.get_widget ("textview1").get_buffer()
			text = text.set_text("\r\n"+self.LIsp+"\r\n\r\n"+self.LNetworkError)




	def clear(self,widget):
		text = self.xml.get_widget ("textview1").get_buffer()
		text = text.set_text("")




	def apply(self,widget):
		try:
			entry1 = self.xml.get_widget ("entry1")
			entry1_text = entry1.get_text()
			self.device_open()
			self.device.write(entry1_text+"\r")
			self.device.readline()
			entry1_output = self.device.readline()
			self.device.close()
			text = self.xml.get_widget ("textview1").get_buffer()
			text = text.set_text("\r\n"+self.LModem+entry1_output)
		except:
			pass
			text = self.xml.get_widget ("textview1").get_buffer()
			text = text.set_text("\r\n"+self.LModem+"\r\n\r\n"+self.LNetworkError)




if __name__ == "__main__":
	kickstarter()
	gtk.main()
